/* Audio Library for Teensy 3.X
   Copyright (c) 2014, Pete (El Supremo)
   Copyright (c) 2019, Holger Wirtz

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#include <Arduino.h>
#include "limits.h"
#include "effect_modulated_chorus.h"
#include "interpolation.h"
#include "config.h"

/******************************************************************/

// Based on;      A u d i o E f f e c t C h o r u s
// Written by Pete (El Supremo) Jan 2014
// 140529 - change to handle mono stream - change modify() to voices()
// 140219 - correct storage class (not static)
// 190527 - added modulation input handling (by Holger Wirtz)

boolean AudioEffectModulatedDelay::begin(short *delayline, int d_length)
{
#if 0
  Serial.print("AudioEffectModulatedDelay.begin(Chorus delay line length = ");
  Serial.print(d_length);
  Serial.println(")");
#endif

  _delayline = NULL;
  _delay_length = 0;
  _circ_idx = 0;

  if (delayline == NULL) {
    return (false);
  }
  if (d_length < 10) {
    return (false);
  }

  _delayline = delayline;
  _delay_length = d_length;
  _delay_length_half = d_length / 2;

  memset(_delayline, 0, sizeof(int16_t)*_delay_length);

  return (true);
}

//int last_idx = 0;
void AudioEffectModulatedDelay::update(void)
{
  audio_block_t *block;
  audio_block_t *modulation;

  int16_t *bp;
  int16_t *mp;
  float mod_idx;

  if (_delayline == NULL)
    return;

#ifdef INTERPOLATE
  interpolation modulation_interpolate;
#endif

  block = receiveWritable(0);
  modulation = receiveReadOnly(1);

#ifdef INTERPOLATE
  int8_t j;
  float x[INTERPOLATION_WINDOW_SIZE];
  float y[INTERPOLATION_WINDOW_SIZE];
  modulation_interpolate.valuelenXY(INTERPOLATION_WINDOW_SIZE);
  modulation_interpolate.valueX(x);
  modulation_interpolate.valueY(y);

  for (j = 0; j < INTERPOLATION_WINDOW_SIZE; j++)
    x[j] = float(j);
#endif

  bp = block->data;
  mp = modulation->data;

  if (block && modulation)
  {
    for (uint16_t i = 0; i < AUDIO_BLOCK_SAMPLES; i++)
    {
      // write data into circular buffer
      if (_circ_idx >= _delay_length)
        _circ_idx = 0;
      _delayline[_circ_idx] = *bp;

      // calculate modulation index
      mod_idx = float(*mp) / SHRT_MAX * _delay_length_half + _circ_idx; // calculate index with modulation as a float(!!!)
      if (mod_idx > float(_delay_length - 1))
        mod_idx = mod_idx - float(_delay_length - 1);
      else if (mod_idx < 0.0)
        mod_idx = float(_delay_length - 1) + mod_idx;

#ifdef INTERPOLATE
      // get x/y values around mod_idx
      uint16_t i_mod_idx = int(mod_idx + 0.5);
      uint8_t c = 0;
      for (j = INTERPOLATION_WINDOW_SIZE / -2; j <= INTERPOLATION_WINDOW_SIZE / 2; j++)
      {
        int16_t ji_mod_idx = i_mod_idx + j;
        if (ji_mod_idx > _delay_length)
          y[c] = float(_delayline[ji_mod_idx - _delay_length - 1]);
        else if (ji_mod_idx < 0)
          y[c] = float(_delayline[_delay_length + j + 1]);
        else
          y[c] = float(_delayline[ji_mod_idx]);
        c++; // ;-)
      }

      modulation_interpolate.valueI(mod_idx);

#if INTERPOLATE == CUBIC
      *bp = int(modulation_interpolate.CubicInterpolate() + 0.5);
#elif INTERPOLATE == LINEAR
      *bp = int(modulation_interpolate.LinearInterpolate() + 0.5);
#elif INTERPOLATE == COSINE
      *bp = int(modulation_interpolate.CosineInterpolate() + 0.5);
#elif INTERPOLATE == LAGRANGE
      *bp = int(modulation_interpolate.LagrangeInterpolate() + 0.5);
#elif INTERPOLATE == QUDRATIC
      *bp = int(modulation_interpolate.QuadraticInterpolate() + 0.5);
#else
      // No interpolation - should sound really bad...
      *bp = _delayline[int(mod_idx + 0.5)];
#endif
#else
      // No interpolation - should sound really bad...
      *bp = _delayline[int(mod_idx + 0.5)];
#endif

      bp++;
      mp++;
      _circ_idx++;
    }
  }

  if (block)
  {
    transmit(block, 0);
    release(block);
  }

  if (modulation)
    release(modulation);
}
